@@ -3,7 +3,7 @@
  *  Prototypes for all individual checks. */
 
 /*
- *  GiNaC Copyright (C) 1999 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,7 +38,8 @@ unsigned differentiation();
 unsigned poly_gcd();
 unsigned normalization();
 unsigned matrix_checks();
-unsigned lsolve_onedim();
+unsigned linear_solve();
 unsigned series_expansion();
+unsigned lortensor_check();
 
 #endif // ndef CHECK_H
