@@ -15,13 +15,16 @@
 #define	T_MATRIX_BEGIN	269
 #define	T_MATRIX_END	270
 #define	T_QUIT	271
-#define	T_PRINT	272
-#define	T_TIME	273
-#define	T_XYZZY	274
-#define	T_INVENTORY	275
-#define	T_LOOK	276
-#define	T_SCORE	277
-#define	NEG	278
+#define	T_WARRANTY	272
+#define	T_PRINT	273
+#define	T_READ	274
+#define	T_WRITE	275
+#define	T_TIME	276
+#define	T_XYZZY	277
+#define	T_INVENTORY	278
+#define	T_LOOK	279
+#define	T_SCORE	280
+#define	NEG	281
 
 
 extern YYSTYPE yylval;
